@@ -25,6 +25,7 @@
 #ifdef ENABLE_ACTIVESYNC
 
 #include "ActiveSyncSource.h"
+#include <syncevo/IdentityProvider.h>
 
 #include <eas-errors.h>
 
@@ -32,6 +33,9 @@
 #include <errno.h>
 
 #include <boost/algorithm/string.hpp>
+#include <boost/range/adaptors.hpp>
+
+SE_GOBJECT_TYPE(EasSyncHandler)
 
 /* #include <eas-connection-errors.h> */
 #include <syncevo/declarations.h>
@@ -39,6 +43,7 @@ SE_BEGIN_CXX
 
 void EASItemUnref(EasItemInfo *info) { g_object_unref(&info->parent_instance); }
 void GStringUnref(char *str) { g_free(str); }
+void EASFolderUnref(EasFolder *f) { g_object_unref(&f->parent_instance); }
 
 void ActiveSyncSource::enableServerMode()
 {
@@ -50,26 +55,185 @@ bool ActiveSyncSource::serverModeEnabled() const
     return m_operations.m_loadAdminData;
 }
 
+/* Recursively work out full path name */
+std::string ActiveSyncSource::Collection::fullPath() {
+    if (!pathFound) {
+	if (parentId == "0") {
+	    pathName = name;
+	} else {
+	    pathName = source->m_collections[parentId].fullPath() + "/" + name;
+	}
+	pathFound = true;
+    }
+
+    return pathName;
+}
+
+void ActiveSyncSource::findCollections(const std::string &account, const bool force_update)
+{
+    GErrorCXX gerror;
+    EasSyncHandlerCXX handler;
+    EASFoldersCXX folders;
+    
+    if (!m_collections.empty()) {
+	if (!force_update) return;
+	m_collections.clear();
+	m_folderPaths.clear();
+    }
+    
+    /* Fetch the folders */
+    handler = EasSyncHandlerCXX::steal(eas_sync_handler_new(account.c_str()));
+    if (!handler) throwError("findCollections cannot allocate sync handler");
+    
+    if (!eas_sync_handler_get_folder_list (handler,
+					   force_update,
+					   folders,
+					   NULL,
+					   gerror)) {
+	gerror.throwError("fetching folder list");
+    }
+    
+    /* Save the Collections */
+    BOOST_FOREACH(EasFolder *folder, folders) {
+	m_collections[folder->folder_id].collectionId = folder->folder_id;
+	m_collections[folder->folder_id].name = folder->display_name;
+	m_collections[folder->folder_id].parentId = folder->parent_id;
+	m_collections[folder->folder_id].type = folder->type;
+	m_collections[folder->folder_id].source = this;
+    }
+    
+    /* Save the full paths */
+    BOOST_FOREACH(std::string id, m_collections | boost::adaptors::map_keys) {
+	m_folderPaths[m_collections[id].fullPath()] = id;
+    }
+}
+
+int ActiveSyncSource::Collection::getFolderType () {
+    switch (type) {
+    case EAS_FOLDER_TYPE_DEFAULT_INBOX:
+    case EAS_FOLDER_TYPE_DEFAULT_DRAFTS:
+    case EAS_FOLDER_TYPE_DEFAULT_DELETED_ITEMS:
+    case EAS_FOLDER_TYPE_DEFAULT_SENT_ITEMS:
+    case EAS_FOLDER_TYPE_DEFAULT_OUTBOX:
+    case EAS_FOLDER_TYPE_USER_CREATED_MAIL:
+	return EAS_ITEM_MAIL;
+    case EAS_FOLDER_TYPE_DEFAULT_TASKS:
+    case EAS_FOLDER_TYPE_USER_CREATED_TASKS:
+	return EAS_ITEM_TODO;
+    case EAS_FOLDER_TYPE_DEFAULT_CALENDAR:
+    case EAS_FOLDER_TYPE_USER_CREATED_CALENDAR:
+	return EAS_ITEM_CALENDAR;
+    case EAS_FOLDER_TYPE_DEFAULT_CONTACTS:
+    case EAS_FOLDER_TYPE_USER_CREATED_CONTACTS:
+	return EAS_ITEM_CONTACT;
+    case EAS_FOLDER_TYPE_DEFAULT_NOTES:
+    case EAS_FOLDER_TYPE_USER_CREATED_NOTES:
+	//TODO: implement memos
+    case EAS_FOLDER_TYPE_DEFAULT_JOURNAL:
+    case EAS_FOLDER_TYPE_USER_CREATED_JOURNAL:
+    case EAS_FOLDER_TYPE_UNKNOWN:
+    case EAS_FOLDER_TYPE_RECIPIENT_CACHE:
+    default:
+	return -1;
+    }
+}
+
+bool ActiveSyncSource::Collection::collectionIsDefault () {
+    return type == EAS_FOLDER_TYPE_DEFAULT_INBOX ||
+	type == EAS_FOLDER_TYPE_DEFAULT_DRAFTS ||
+	type == EAS_FOLDER_TYPE_DEFAULT_DELETED_ITEMS ||
+	type == EAS_FOLDER_TYPE_DEFAULT_SENT_ITEMS ||
+	type == EAS_FOLDER_TYPE_DEFAULT_OUTBOX ||
+	type == EAS_FOLDER_TYPE_DEFAULT_TASKS ||
+	type == EAS_FOLDER_TYPE_DEFAULT_CALENDAR ||
+	type == EAS_FOLDER_TYPE_DEFAULT_CONTACTS ||
+	type == EAS_FOLDER_TYPE_DEFAULT_NOTES ||
+	type == EAS_FOLDER_TYPE_DEFAULT_JOURNAL;
+}
+
 ActiveSyncSource::Databases ActiveSyncSource::getDatabases()
 {
     Databases result;
-    // empty string always selects the default database
-    result.push_back(Database("", "", true));
+    // do a scan if username is set
+    UserIdentity identity = m_context->getSyncUser();
+    if (identity.m_provider != USER_IDENTITY_PLAIN_TEXT) {
+        throwError(StringPrintf("%s: only the 'user:<account ID in gconf>' format is supported by ActiveSync", identity.toString().c_str()));
+    }
+    const std::string &account = identity.m_identity;
+
+    if (!account.empty()) {
+
+	findCollections(account, true);
+
+	BOOST_FOREACH(Collection coll, m_collections | boost::adaptors::map_values) {
+	    if (coll.getFolderType() == getEasType()) {
+		result.push_back(Database(coll.pathName, coll.collectionId, coll.collectionIsDefault()));
+	    }
+	}
+
+    } else {
+	result.push_back(Database("to scan, specify --print-databases username=<account> backend=\""+getSourceType().m_backend+"\"",
+                                  ""));
+    }
+
     return result;
 }
 
+std::string ActiveSyncSource::lookupFolder(const std::string &folder) {
+    // If folder matches a collectionId, use that
+    if (m_collections.find(folder) != m_collections.end()) return folder;
+
+    // If folder begins with /, drop it
+    std::string key;
+    if (!folder.empty() && folder[0] == '/') {
+        key = folder.substr(1);
+    } else {
+        key = folder;
+    }
+
+    // Lookup folder name
+    FolderPaths::const_iterator entry = m_folderPaths.find(key);
+    if (entry != m_folderPaths.end()) {
+        return entry->second;
+    }
+
+    // Not found
+    return "";
+}
 
 void ActiveSyncSource::open()
 {
     // extract account ID and throw error if missing
-    std::string username = m_context->getSyncUsername();
-    SE_LOG_DEBUG(NULL, NULL,
-                 "using eas sync account %s from config %s",
+    UserIdentity identity = m_context->getSyncUser();
+    if (identity.m_provider != USER_IDENTITY_PLAIN_TEXT) {
+        throwError(StringPrintf("%s: only the 'user:<account ID in gconf>' format is supported by ActiveSync", identity.toString().c_str()));
+    }
+    const std::string &username = identity.m_identity;
+
+    std::string folder = getDatabaseID();
+    SE_LOG_DEBUG(NULL,
+                 "using eas sync account %s from config %s with folder %s",
                  username.c_str(),
-                 m_context->getConfigName().c_str());
+                 m_context->getConfigName().c_str(),
+		 folder.c_str());
+
+    if (folder.empty()) { // Most common case is empty string
+	m_folder = folder;
+    } else { // Lookup folder name
+	// Try using cached folder list
+	findCollections(username, false);
+	m_folder = lookupFolder(folder);
+	if (m_folder.empty()) {
+	    // Fetch latest folder list and try again
+	    findCollections(username, true);
+	    m_folder = lookupFolder(folder);
+	}
+	if (m_folder.empty()) {
+	    throwError("could not find folder: "+folder);
+	}
+    }
 
     m_account = username;
-    m_folder = getDatabaseID();
 
     // create handler
     m_handler.set(eas_sync_handler_new(m_account.c_str()), "EAS handler");
@@ -95,10 +259,10 @@ void ActiveSyncSource::beginSync(const std::string &lastToken, const std::string
     m_startSyncKey = lastToken;
     if (lastToken.empty()) {
         // slow sync: wipe out cached list of IDs, will be filled anew below
-        SE_LOG_DEBUG(this, NULL, "sync key empty, starting slow sync");
+        SE_LOG_DEBUG(getDisplayName(), "sync key empty, starting slow sync");
         m_ids->clear();
     } else {
-        SE_LOG_DEBUG(this, NULL, "sync key %s for account '%s' folder '%s', starting incremental sync",
+        SE_LOG_DEBUG(getDisplayName(), "sync key %s for account '%s' folder '%s', starting incremental sync",
                      lastToken.c_str(),
                      m_account.c_str(),
                      m_folder.c_str());
@@ -159,7 +323,7 @@ void ActiveSyncSource::beginSync(const std::string &lastToken, const std::string
             if (luid.empty()) {
                 throwError("empty server ID for new eas item");
             }
-            SE_LOG_DEBUG(this, NULL, "new item %s", luid.c_str());
+            SE_LOG_DEBUG(getDisplayName(), "new item %s", luid.c_str());
             addItem(luid, NEW);
             m_ids->setProperty(luid, "1");
             if (!item->data) {
@@ -175,7 +339,7 @@ void ActiveSyncSource::beginSync(const std::string &lastToken, const std::string
             if (luid.empty()) {
                 throwError("empty server ID for updated eas item");
             }
-            SE_LOG_DEBUG(this, NULL, "updated item %s", luid.c_str());
+            SE_LOG_DEBUG(getDisplayName(), "updated item %s", luid.c_str());
             addItem(luid, UPDATED);
             // m_ids.setProperty(luid, "1"); not necessary, should already exist (TODO: check?!)
             if (!item->data) {
@@ -191,7 +355,7 @@ void ActiveSyncSource::beginSync(const std::string &lastToken, const std::string
             if (luid.empty()) {
                 throwError("empty server ID for deleted eas item");
             }
-            SE_LOG_DEBUG(this, NULL, "deleted item %s", luid.c_str());
+            SE_LOG_DEBUG(getDisplayName(), "deleted item %s", luid.c_str());
             addItem(luid, DELETED);
             m_ids->removeProperty(luid);
         }
@@ -217,7 +381,7 @@ void ActiveSyncSource::beginSync(const std::string &lastToken, const std::string
     m_ids->readProperties(props);
     BOOST_FOREACH(const StringPair &entry, props) {
         const std::string &luid = entry.first;
-        SE_LOG_DEBUG(this, NULL, "existing item %s", luid.c_str());
+        SE_LOG_DEBUG(getDisplayName(), "existing item %s", luid.c_str());
         addItem(luid, ANY);
     }
 
@@ -240,7 +404,7 @@ std::string ActiveSyncSource::endSync(bool success)
     // let engine do incremental sync next time or start from scratch
     // in case of failure
     std::string newSyncKey = success ? m_currentSyncKey : "";
-    SE_LOG_DEBUG(this, NULL, "next sync key %s", newSyncKey.empty() ? "empty" : newSyncKey.c_str());
+    SE_LOG_DEBUG(getDisplayName(), "next sync key %s", newSyncKey.empty() ? "empty" : newSyncKey.c_str());
     return newSyncKey;
 }
 
