@@ -156,7 +156,7 @@ SyncSource::Databases EvolutionCalendarSource::getDatabases()
             throwError("unable to access backend databases", gerror);
         }
     }
-    ESourceListCXX sources(tmp, false);
+    ESourceListCXX sources(tmp, TRANSFER_REF);
     bool first = true;
     for (GSList *g = sources ? e_source_list_peek_groups (sources) : NULL;
          g;
@@ -197,7 +197,7 @@ char *EvolutionCalendarSource::authenticate(const char *prompt,
 {
     std::string passwd = getPassword();
 
-    SE_LOG_DEBUG(this, NULL, "authentication requested, prompt \"%s\", key \"%s\" => %s",
+    SE_LOG_DEBUG(getDisplayName(), "authentication requested, prompt \"%s\", key \"%s\" => %s",
                  prompt, key,
                  !passwd.empty() ? "returning configured password" : "no password configured");
     return !passwd.empty() ? strdup(passwd.c_str()) : NULL;
@@ -240,7 +240,7 @@ void EvolutionCalendarSource::open()
     if (!e_cal_get_sources(&tmp, sourceType(), gerror)) {
         throwError("unable to access backend databases", gerror);
     }
-    ESourceListCXX sources(tmp, false);
+    ESourceListCXX sources(tmp, TRANSFER_REF);
 
     string id = getDatabaseID();    
     ESource *source = findSource(sources, id);
@@ -307,15 +307,23 @@ bool EvolutionCalendarSource::isEmpty()
 #ifdef USE_EDS_CLIENT
 class ECalClientViewSyncHandler {
   public:
-    ECalClientViewSyncHandler(ECalClientView *view, void (*processList)(const GSList *list, void *user_data), void *user_data): 
-        m_processList(processList), m_userData(user_data), m_view(view)
+    typedef boost::function<void(const GSList *list)> Process_t;
+
+    ECalClientViewSyncHandler(ECalClientViewCXX &view,
+                              const Process_t &process) :
+        m_process(process),
+        m_view(view)
     {}
 
     bool processSync(GErrorCXX &gerror)
     {
         // Listen for view signals
-        g_signal_connect(m_view, "objects-added", G_CALLBACK(objectsAdded), this);
-        g_signal_connect(m_view, "complete", G_CALLBACK(completed), this);
+        m_view.connectSignal<void (ECalClientView *ebookview,
+                                   const GSList *contacts)>("objects-added",
+                                                            boost::bind(m_process, _2));
+        m_view.connectSignal<void (EBookClientView *ebookview,
+                                   const GError *error)>("complete",
+                                                         boost::bind(&ECalClientViewSyncHandler::completed, this, _2));
 
         // Start the view
         e_cal_client_view_start (m_view, m_error);
@@ -335,41 +343,30 @@ class ECalClientViewSyncHandler {
             return true;
         }
     }
- 
-    static void objectsAdded(ECalClientView *ebookview,
-                             const GSList *objects,
-                             gpointer user_data) {
-        ECalClientViewSyncHandler *that = (ECalClientViewSyncHandler *)user_data;
-        that->m_processList(objects, that->m_userData);
+
+    void completed(const GError *error)
+    {
+        m_error = error;
+        m_loop.quit();
     }
- 
-    static void completed(ECalClientView *ebookview,
-                          const GError *error,
-                          gpointer user_data) {
-        ECalClientViewSyncHandler *that = (ECalClientViewSyncHandler *)user_data;
-        that->m_error = error;
-        that->m_loop.quit();
-    }
- 
+
     public:
-      // Process list callback
-      void (*m_processList)(const GSList *list, void *user_data);
-      void *m_userData;
       // Event loop for Async -> Sync
       EvolutionAsync m_loop;
 
     private:
+      // Process list callback
+      Process_t m_process;
+
       // View watched
-      ECalClientView *m_view;
+      ECalClientViewCXX m_view;
 
       // Possible error while watching the view
       GErrorCXX m_error;
 };
 
-static void list_revisions(const GSList *objects, void *user_data)
+static void list_revisions(const GSList *objects, EvolutionCalendarSource::RevisionMap_t *revisions)
 {
-    EvolutionCalendarSource::RevisionMap_t *revisions = 
-        static_cast<EvolutionCalendarSource::RevisionMap_t *>(user_data);
     const GSList *l;
 
     for (l = objects; l; l = l->next) {
@@ -396,7 +393,7 @@ void EvolutionCalendarSource::listAllItems(RevisionMap_t &revisions)
 
     // TODO: Optimization: use set fields_of_interest (UID / REV / LAST-MODIFIED)
 
-    ECalClientViewSyncHandler handler(viewPtr, list_revisions, &revisions);
+    ECalClientViewSyncHandler handler(viewPtr, boost::bind(list_revisions, _1, &revisions));
     if (!handler.processSync(gerror)) {
         throwError("watching view", gerror);
     }
@@ -404,7 +401,7 @@ void EvolutionCalendarSource::listAllItems(RevisionMap_t &revisions)
     // Update m_allLUIDs
     m_allLUIDs.clear();
     RevisionMap_t::iterator it;
-    for(it = revisions.begin(); it != revisions.end(); it++) {
+    for(it = revisions.begin(); it != revisions.end(); ++it) {
         m_allLUIDs.insertLUID(it->first);
     }
 #else
@@ -433,7 +430,7 @@ void EvolutionCalendarSource::listAllItems(RevisionMap_t &revisions)
 
 void EvolutionCalendarSource::close()
 {
-    m_calendar = NULL;
+    m_calendar.reset();
 }
 
 void EvolutionCalendarSource::readItem(const string &luid, std::string &item, bool raw)
@@ -504,7 +501,7 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
         propstart = data.find("\nCATEGORIES", propstart + 1);
     }
     if (modified) {
-        SE_LOG_DEBUG(this, NULL, "after replacing , with \\, in CATEGORIES:\n%s", data.c_str());
+        SE_LOG_DEBUG(getDisplayName(), "after replacing , with \\, in CATEGORIES:\n%s", data.c_str());
     }
 
     eptr<icalcomponent> icomp(icalcomponent_new_from_string((char *)data.c_str()));
@@ -548,7 +545,7 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
         const char *tzid = icaltimezone_get_tzid(zone);
         if (!tzid || !tzid[0]) {
             // cannot add a VTIMEZONE without TZID
-            SE_LOG_DEBUG(this, NULL, "skipping VTIMEZONE without TZID");
+            SE_LOG_DEBUG(getDisplayName(), "skipping VTIMEZONE without TZID");
         } else {
             gboolean success =
 #ifdef USE_EDS_CLIENT
@@ -822,7 +819,7 @@ EvolutionCalendarSource::ICalComps_t EvolutionCalendarSource::removeEvents(const
 
     // removes all events with that UID, including children
     GErrorCXX gerror;
-    if (
+    if (!uid.empty() && // e_cal_client_remove_object_sync() in EDS 3.8 aborts the process for empty UID, other versions cannot succeed, so skip the call.
 #ifdef USE_EDS_CLIENT
         !e_cal_client_remove_object_sync(m_calendar,
                                          uid.c_str(), NULL, CALOBJ_MOD_ALL,
@@ -835,7 +832,7 @@ EvolutionCalendarSource::ICalComps_t EvolutionCalendarSource::removeEvents(const
 #endif
         ) {
         if (IsCalObjNotFound(gerror)) {
-            SE_LOG_DEBUG(this, NULL, "%s: request to delete non-existant item ignored",
+            SE_LOG_DEBUG(getDisplayName(), "%s: request to delete non-existant item ignored",
                          uid.c_str());
             if (!ignoreNotFound) {
                 throwError(STATUS_NOT_FOUND, string("delete item: ") + uid);
@@ -861,7 +858,7 @@ void EvolutionCalendarSource::removeItem(const string &luid)
          * remove all items with the given uid and if we only wanted to
          * delete the parent, then recreate the children.
          */
-        ICalComps_t children = removeEvents(id.m_uid, true, false);
+        ICalComps_t children = removeEvents(id.m_uid, true, TRANSFER_REF);
 
         // recreate children
         bool first = true;
@@ -923,7 +920,7 @@ void EvolutionCalendarSource::removeItem(const string &luid)
             ;
         if (!item ||
             (!success && IsCalObjNotFound(gerror))) {
-            SE_LOG_DEBUG(this, NULL, "%s: request to delete non-existant item",
+            SE_LOG_DEBUG(getDisplayName(), "%s: request to delete non-existant item",
                          luid.c_str());
             throwError(STATUS_NOT_FOUND, string("delete item: ") + id.getLUID());
         } else if (!success) {
@@ -1037,7 +1034,7 @@ string EvolutionCalendarSource::retrieveItemAsString(const ItemID &id)
         if (!icalstr) {
             throwError(string("could not encode item as iCalendar: ") + id.getLUID());
         } else {
-            SE_LOG_DEBUG(this, NULL, "had to remove TZIDs because e_cal_get_component_as_string() failed for:\n%s", icalstr.get());
+            SE_LOG_DEBUG(getDisplayName(), "had to remove TZIDs because e_cal_get_component_as_string() failed for:\n%s", icalstr.get());
 	}
     }
 
@@ -1069,7 +1066,7 @@ string EvolutionCalendarSource::retrieveItemAsString(const ItemID &id)
         propstart = data.find("\nCATEGORIES", propstart + 1);
     }
     if (modified) {
-        SE_LOG_DEBUG(this, NULL, "after replacing \\, with , in CATEGORIES:\n%s", data.c_str());
+        SE_LOG_DEBUG(getDisplayName(), "after replacing \\, with , in CATEGORIES:\n%s", data.c_str());
     }
     
     return data;
@@ -1179,6 +1176,9 @@ string EvolutionCalendarSource::getItemModTime(ECalComponent *ecomp)
 
 string EvolutionCalendarSource::getItemModTime(const ItemID &id)
 {
+    if (!needChanges()) {
+        return "";
+    }
     eptr<icalcomponent> icomp(retrieveItem(id));
     return getItemModTime(icomp);
 }
