@@ -98,7 +98,7 @@ public:
     {
         foreach (KCalCore::Incidence::Ptr incidence, incidences) {
             if (incidence->type() == m_type) {
-                SE_LOG_DEBUG(NULL, NULL, "item %s %s",
+                SE_LOG_DEBUG(NULL, "item %s %s",
                              getItemID(incidence).getLUID().c_str(),
                              state == SyncSourceChanges::ANY ? "exists" :
                              state == SyncSourceChanges::NEW ? "is new" :
@@ -421,7 +421,7 @@ void KCalExtendedSource::beginSync(const std::string &lastToken, const std::stri
     // }
     m_data->extractIncidences(incidences, SyncSourceChanges::ANY, *this);
     if (*anchor) {
-        SE_LOG_DEBUG(NULL, NULL, "checking for changes since %s UTC", anchor);
+        SE_LOG_DEBUG(NULL, "checking for changes since %s UTC", anchor);
         KDateTime endSyncTime(QDateTime::fromString(QString(anchor), Qt::ISODate), KDateTime::Spec::UTC());
         KCalCore::Incidence::List added, modified, deleted;
         if (!m_data->m_storage->insertedIncidences(&added, endSyncTime, m_data->m_notebookUID)) {
