@@ -25,6 +25,7 @@
 #endif
 
 #include <boost/bind.hpp>
+#include <set>
 
 #include <string.h>
 
@@ -153,30 +154,19 @@ GLibSelectResult GLibSelect(GMainLoop *loop, int fd, int direction, Timespec *ti
     return instance.run();
 }
 
-void GLibErrorException(const string &action, GError *gerror)
-{
-    string gerrorstr = action;
-    if (!gerrorstr.empty()) {
-        gerrorstr += ": ";
-    }
-    if (gerror) {
-        gerrorstr += gerror->message;
-        g_clear_error(&gerror);
-    } else {
-        gerrorstr = "failure";
-    }
-
-    SE_THROW(gerrorstr);
-}
-
 void GErrorCXX::throwError(const string &action)
+{
+    throwError(action, m_gerror);
+}
+
+void GErrorCXX::throwError(const string &action, const GError *err)
 {
     string gerrorstr = action;
     if (!gerrorstr.empty()) {
         gerrorstr += ": ";
     }
-    if (m_gerror) {
-        gerrorstr += m_gerror->message;
+    if (err) {
+        gerrorstr += err->message;
         // No need to clear m_error! Will be done as part of
         // destructing the GErrorCCXX.
     } else {
@@ -202,12 +192,12 @@ GLibNotify::GLibNotify(const char *file,
                        const callback_t &callback) :
     m_callback(callback)
 {
-    GFileCXX filecxx(g_file_new_for_path(file));
-    GError *error = NULL;
-    GFileMonitorCXX monitor(g_file_monitor_file(filecxx.get(), G_FILE_MONITOR_NONE, NULL, &error));
+    GFileCXX filecxx(g_file_new_for_path(file), TRANSFER_REF);
+    GErrorCXX gerror;
+    GFileMonitorCXX monitor(g_file_monitor_file(filecxx.get(), G_FILE_MONITOR_NONE, NULL, gerror), TRANSFER_REF);
     m_monitor.swap(monitor);
     if (!m_monitor) {
-        GLibErrorException(std::string("monitoring ") + file, error);
+        gerror.throwError(std::string("monitoring ") + file);
     }
     g_signal_connect_after(m_monitor.get(),
                            "changed",
@@ -215,6 +205,93 @@ GLibNotify::GLibNotify(const char *file,
                            (void *)&m_callback);
 }
 
+class PendingChecks
+{
+    typedef std::set<const boost::function<bool ()> *> Checks;
+    Checks m_checks;
+    DynMutex m_mutex;
+    Cond m_cond;
+
+public:
+    /**
+     * Called by main thread before and after sleeping.
+     * Runs all registered checks and removes the ones
+     * which are done.
+     */
+    void runChecks();
+
+    /**
+     * Called by additional threads. Returns when check()
+     * returned false.
+     */
+    void blockOnCheck(const boost::function<bool ()> &check);
+};
+
+void PendingChecks::runChecks()
+{
+    DynMutex::Guard guard = m_mutex.lock();
+    Checks::iterator it = m_checks.begin();
+    bool removed = false;
+    while (it != m_checks.end()) {
+        bool cont;
+        try {
+            cont = (**it)();
+        } catch (...) {
+            Exception::handle(HANDLE_EXCEPTION_FATAL);
+            // keep compiler happy
+            cont = false;
+        }
+
+        if (!cont) {
+            // Done with this check
+            Checks::iterator next = it;
+            ++next;
+            m_checks.erase(it);
+            it = next;
+            removed = true;
+        } else {
+            ++it;
+        }
+    }
+    // Tell blockOnCheck() calls that they may have completed.
+    if (removed) {
+        m_cond.signal();
+    }
+}
+
+void PendingChecks::blockOnCheck(const boost::function<bool ()> &check)
+{
+    DynMutex::Guard guard = m_mutex.lock();
+    // When we get here, the conditions for returning may already have
+    // been met.  Check before sleeping. If we need to continue, then
+    // holding the mutex ensures that the main thread will run the
+    // check on the next iteration.
+    if (check()) {
+        m_checks.insert(&check);
+        do {
+             m_cond.wait(m_mutex);
+        } while (m_checks.find(&check) != m_checks.end());
+    }
+}
+
+void GRunWhile(const boost::function<bool ()> &check)
+{
+    static PendingChecks checks;
+    if (g_main_context_is_owner(g_main_context_default())) {
+        // Check once before sleeping, conditions may already be met
+        // for some checks.
+        checks.runChecks();
+        // Drive event loop.
+        while (check()) {
+            g_main_context_iteration(NULL, true);
+            checks.runChecks();
+        }
+    } else {
+        // Transfer check into main thread.
+        checks.blockOnCheck(check);
+    }
+}
+
 #ifdef ENABLE_UNIT_TESTS
 
 class GLibTest : public CppUnit::TestFixture {
@@ -251,7 +328,7 @@ class GLibTest : public CppUnit::TestFixture {
         list<Event> events;
         static const char *name = "GLibTest.out";
         unlink(name);
-        GMainLoopCXX loop(g_main_loop_new(NULL, FALSE), false);
+        GMainLoopCXX loop(g_main_loop_new(NULL, FALSE), TRANSFER_REF);
         if (!loop) {
             SE_THROW("could not allocate main loop");
         }
@@ -263,7 +340,7 @@ class GLibTest : public CppUnit::TestFixture {
             out << "hello";
             out.close();
             g_main_loop_run(loop.get());
-            CPPUNIT_ASSERT(events.size() > 0);
+            CPPUNIT_ASSERT(!events.empty());
         }
 
         {
@@ -272,7 +349,7 @@ class GLibTest : public CppUnit::TestFixture {
             out.close();
             GLibEvent id(g_timeout_add_seconds(5, timeout, loop.get()), "timeout");
             g_main_loop_run(loop.get());
-            CPPUNIT_ASSERT(events.size() > 0);
+            CPPUNIT_ASSERT(!events.empty());
         }
 
         {
@@ -280,7 +357,7 @@ class GLibTest : public CppUnit::TestFixture {
             unlink(name);
             GLibEvent id(g_timeout_add_seconds(5, timeout, loop.get()), "timeout");
             g_main_loop_run(loop.get());
-            CPPUNIT_ASSERT(events.size() > 0);
+            CPPUNIT_ASSERT(!events.empty());
         }
     }
 };
