@@ -62,6 +62,12 @@ class LocalTransportChild;
  */
 class LocalTransportAgent : public TransportAgent
 {
+ private:
+    LocalTransportAgent(SyncContext *server,
+                        const std::string &clientContext,
+                        void *loop = NULL);
+    boost::weak_ptr<LocalTransportAgent> m_self;
+
  public:
     /**
      * @param server          the server side of the sync;
@@ -71,9 +77,9 @@ class LocalTransportAgent : public TransportAgent
      * @param loop            optional glib loop to use when waiting for IO;
      *                        transport will *not* increase the reference count
      */
-    LocalTransportAgent(SyncContext *server,
-                        const std::string &clientContext,
-                        void *loop = NULL);
+    static boost::shared_ptr<LocalTransportAgent> create(SyncContext *server,
+                                                         const std::string &clientContext,
+                                                         void *loop = NULL);
     ~LocalTransportAgent();
 
     /**
@@ -120,12 +126,13 @@ class LocalTransportAgent : public TransportAgent
      * provides access to the forked process' D-Bus API
      * - start sync (returns child's first message)
      * - send server reply (returns child's next message or empty when done)
+     * - emits output via signal
      *
      * Only non-NULL when child is running and connected.
      */
     boost::shared_ptr<LocalTransportChild> m_child;
 
-
+    void logChildOutput(const std::string &level, const std::string &message);
     void onChildConnect(const GDBusCXX::DBusConnectionPtr &conn);
     void onFailure(const std::string &error);
     void onChildQuit(int status);
