@@ -25,7 +25,6 @@
 
 #include <syncevo/declarations.h>
 SE_BEGIN_CXX
-using namespace std;
 
 class MapSyncSource;
 
@@ -167,6 +166,11 @@ class SubSyncSource : virtual public SyncSourceBase
      */
     virtual std::string removeSubItem(const string &mainid, const std::string &subid) = 0;
 
+    /**
+     * Remove all sub-items belonging to mainid.
+     */
+    virtual void removeMergedItem(const std::string &mainid) = 0;
+
     /**
      * Called whenever this class thinks that the item may no longer be
      * needed. Might be wrong...
@@ -180,6 +184,12 @@ class SubSyncSource : virtual public SyncSourceBase
      */
     virtual std::string getSubDescription(const string &mainid, const string &subid) = 0;
 
+    /**
+     * Called after MapSyncSource already populated the info structure.
+     */
+    virtual void updateSynthesisInfo(SynthesisInfo &info,
+                                     XMLConfigFragments &fragments) {}
+
  private:
     MapSyncSource *m_parent;
 };
@@ -266,6 +276,16 @@ class MapSyncSource :
     virtual std::string getDescription(sysync::KeyH aItemKey) { return dynamic_cast<SyncSourceLogging &>(*m_sub).getDescription(aItemKey); }
     virtual std::string getDescription(const string &luid);
 
+    /* TestingSyncSource */
+    virtual void removeAllItems();
+
+ protected:
+    virtual void getSynthesisInfo(SynthesisInfo &info,
+                                  XMLConfigFragments &fragments) {
+        TestingSyncSource::getSynthesisInfo(info, fragments);
+        m_sub->updateSynthesisInfo(info, fragments);
+    }
+
  private:
     boost::shared_ptr<SubSyncSource> m_sub;
     /** escape / in uid with %2F, so that splitMainIDValue() and splitLUID() can use / as separator */
