@@ -106,7 +106,10 @@ public:
 class ClientListener : public CppUnit::TestListener {
 public:
     ClientListener() :
-        m_failed(false)
+        m_failed(false),
+        // Not really necessary, will be initialized once the test starts.
+        // Set it anyway, to keep cppcheck happy.
+        m_testFailed(false)
     {
 #ifdef HAVE_SIGNAL_H
         // install signal handler which turns an alarm signal into a runtime exception
@@ -123,9 +126,7 @@ public:
     }
 
     ~ClientListener() {
-        if (&LoggerBase::instance() == m_logger.get()) {
-            LoggerBase::popLogger();
-        }
+        m_logger.reset();
     }
 
     void addAllowedFailures(string allowedFailures) {
@@ -138,11 +139,10 @@ public:
         if (!getenv("SYNCEVOLUTION_DEBUG")) {
             string logfile = m_currentTest + ".log";
             simplifyFilename(logfile);
-            m_logger.reset(new LogRedirect(true, logfile.c_str()));
+            m_logger.reset(new LogRedirect(LogRedirect::STDERR_AND_STDOUT, logfile.c_str()));
             m_logger->setLevel(Logger::DEBUG);
-            LoggerBase::pushLogger(m_logger.get());
         }
-        SE_LOG_DEBUG(NULL, NULL, "*** starting %s ***", m_currentTest.c_str());
+        SE_LOG_DEBUG(NULL, "*** starting %s ***", m_currentTest.c_str());
         m_failures.reset();
         m_testFailed = false;
 
@@ -188,12 +188,9 @@ public:
             result = "okay";
         }
 
-        SE_LOG_DEBUG(NULL, NULL, "*** ending %s: %s ***", m_currentTest.c_str(), result.c_str());
+        SE_LOG_DEBUG(NULL, "*** ending %s: %s ***", m_currentTest.c_str(), result.c_str());
         if (!failure.empty()) {
-            SE_LOG_ERROR(NULL, NULL, "%s", failure.c_str());
-        }
-        if (&LoggerBase::instance() == m_logger.get()) {
-            LoggerBase::popLogger();
+            SE_LOG_ERROR(NULL, "%s", failure.c_str());
         }
         m_logger.reset();
 
@@ -244,8 +241,10 @@ private:
     set<string> m_allowedFailures;
     bool m_failed, m_testFailed;
     string m_currentTest;
+#ifdef HAVE_SIGNAL_H
     int m_alarmSeconds;
-    auto_ptr<LoggerBase> m_logger;
+#endif
+    PushLogger<Logger> m_logger;
     CppUnit::TestResultCollector m_failures;
 
     static void alarmTriggered(int signal) {
@@ -335,7 +334,7 @@ int main(int argc, char* argv[])
 
 
   if (getenv("SYNCEVOLUTION_DEBUG")) {
-      LoggerBase::instance().setLevel(Logger::DEBUG);
+      Logger::instance().setLevel(Logger::DEBUG);
   }
 
   try {
